@@ -252,12 +252,21 @@ sub _pack {
         utf8::encode( $value ) if utf8::is_utf8( $value );
 
         my $num = length $value;
-        my $header =
-              $num < 32          ? CORE::pack( 'C',  0xa0 + $num )
-            : $num < 2 ** 16 - 1 ? CORE::pack( 'Cn', 0xda, $num )
-            : $num < 2 ** 32 - 1 ? CORE::pack( 'CN', 0xdb, $num )
-            : _unexpected('number %d', $num)
-        ;
+        my $header;
+        if ($self->{utf8}) { # Str
+            $header =
+                $num < 32          ? CORE::pack( 'C',  0xa0 + $num )
+                : $num < 2 ** 8  - 1 ? CORE::pack( 'CC', 0xd9, $num)
+                : $num < 2 ** 16 - 1 ? CORE::pack( 'Cn', 0xda, $num )
+                : $num < 2 ** 32 - 1 ? CORE::pack( 'CN', 0xdb, $num )
+                : _unexpected('number %d', $num);
+        } else { # Bin
+            $header =
+                $num < 2 ** 8 - 1 ? CORE::pack( 'CC',  0xc4, $num)
+                : $num < 2 ** 16 - 1 ? CORE::pack( 'Cn', 0xc5, $num )
+                : $num < 2 ** 32 - 1 ? CORE::pack( 'CN', 0xc6, $num )
+                : _unexpected('number %d', $num);
+        }
 
         return $header . $value;
 
@@ -308,10 +317,11 @@ sub unpack :method {
     return $data;
 }
 
-my $T_RAW             = 0x01;
+my $T_STR             = 0x01;
 my $T_ARRAY           = 0x02;
 my $T_MAP             = 0x04;
-my $T_DIRECT          = 0x08; # direct mapping (e.g. 0xc0 <-> nil)
+my $T_BIN             = 0x08;
+my $T_DIRECT          = 0x10; # direct mapping (e.g. 0xc0 <-> nil)
 
 my @typemap = ( (0x00) x 256 );
 
@@ -325,10 +335,16 @@ $typemap[$_] |= $T_MAP for
     0xde,         # map16
     0xdf,         # map32
 ;
-$typemap[$_] |= $T_RAW for
-    0xa0 .. 0xbf, # fix raw
-    0xda,         # raw16
-    0xdb,         # raw32
+$typemap[$_] |= $T_STR for
+    0xa0 .. 0xbf, # fix str
+    0xd9,         # str8
+    0xda,         # str16
+    0xdb,         # str32
+;
+$typemap[$_] |= $T_BIN for
+    0xc4,         # bin 8
+    0xc5,         # bin 16
+    0xc6,         # bin 32
 ;
 
 my @byte2value;
@@ -345,8 +361,13 @@ foreach my $pair(
 }
 
 sub _fetch_size {
-    my($value_ref, $byte, $x16, $x32, $x_fixbits) = @_;
-    if ( $byte == $x16 ) {
+    my($value_ref, $byte, $x8, $x16, $x32, $x_fixbits) = @_;
+    if ( defined($x8) && $byte == $x8 ) {
+        $p += 1;
+        $p <= length(${$value_ref}) or _insufficient('x/8');
+        return unpack 'C', substr( ${$value_ref}, $p - 1, 1);
+    }
+    elsif ( $byte == $x16 ) {
         $p += 2;
         $p <= length(${$value_ref}) or _insufficient('x/16');
         return unpack 'n', substr( ${$value_ref}, $p - 2, 2 );
@@ -371,22 +392,22 @@ sub _unpack {
     # +/- fixnum, nil, true, false
     return $byte2value[$byte] if $typemap[$byte] & $T_DIRECT;
 
-    if ( $typemap[$byte] & $T_RAW ) {
-        my $size = _fetch_size(\$value, $byte, 0xda, 0xdb, 0xa0);
+    if ( $typemap[$byte] & $T_STR ) {
+        my $size = _fetch_size(\$value, $byte, 0xd9, 0xda, 0xdb, 0xa0);
         my $s    = substr( $value, $p, $size );
         length($s) == $size or _insufficient('raw');
         $p      += $size;
-        utf8::decode($s) if $_utf8;
+        utf8::decode($s);
         return $s;
     }
     elsif ( $typemap[$byte] & $T_ARRAY ) {
-        my $size = _fetch_size(\$value, $byte, 0xdc, 0xdd, 0x90);
+        my $size = _fetch_size(\$value, $byte, undef, 0xdc, 0xdd, 0x90);
         my @array;
         push @array, _unpack( $value ) while --$size >= 0;
         return \@array;
     }
     elsif ( $typemap[$byte] & $T_MAP ) {
-        my $size = _fetch_size(\$value, $byte, 0xde, 0xdf, 0x80);
+        my $size = _fetch_size(\$value, $byte, undef, 0xde, 0xdf, 0x80);
         my %map;
         while(--$size >= 0) {
             no warnings; # for undef key case
@@ -396,7 +417,14 @@ sub _unpack {
         }
         return \%map;
     }
-
+    elsif ($typemap[$byte] & $T_BIN) {
+        my $size = _fetch_size(\$value, $byte, 0xc4, 0xc5, 0xc6, 0x80);
+        my $s    = substr( $value, $p, $size );
+        length($s) == $size or _insufficient('bin');
+        $p      += $size;
+        utf8::decode($s) if $_utf8;
+        return $s;
+    }
     elsif ( $byte == 0xcc ) { # uint8
         $p++;
         $p <= length($value) or _insufficient('uint8');
@@ -525,9 +553,13 @@ sub _count {
     # +/- fixnum, nil, true, false
     return 1 if $typemap[$byte] & $T_DIRECT;
 
-    if ( $typemap[$byte] & $T_RAW ) {
+    if ( $typemap[$byte] & $T_STR ) {
         my $num;
-        if ( $byte == 0xda ) {
+        if ( $byte == 0xd9 ) {
+            $num = unpack 'C', substr( $value, $p, 1 );
+            $p += 1;
+        }
+        elsif ( $byte == 0xda ) {
             $num = unpack 'n', substr( $value, $p, 2 );
             $p += 2;
         }
@@ -581,7 +613,24 @@ sub _count {
 
         return 1;
     }
+    elsif ( $typemap[$byte] & $T_BIN ) {
+        my $num;
+        if ( $byte == 0xc4 ) { # bin 8
+            $num = unpack 'C', substr( $value, $p, 1 );
+            $p += 1;
+        }
+        elsif ( $byte == 0xc5 ) { # bin 16
+            $num = unpack 'n', substr( $value, $p, 2 );
+            $p += 2;
+        }
+        elsif ( $byte == 0xc6 ) { # bin 32
+            $num = unpack 'N', substr( $value, $p, 4 );
+            $p += 4;
+        }
 
+        $p += $num;
+        return 1;
+    }
     elsif ( $byte >= 0xcc and $byte <= 0xcf ) { # uint
         $p += $byte == 0xcc ? 1
             : $byte == 0xcd ? 2
