@@ -257,17 +257,32 @@ STATIC_INLINE int template_callback_map_item(unpack_user* u PERL_UNUSED_DECL, SV
     return 0;
 }
 
-STATIC_INLINE int template_callback_raw(unpack_user* u PERL_UNUSED_DECL, const char* b PERL_UNUSED_DECL, const char* p, unsigned int l, SV** o)
+STATIC_INLINE int template_callback_str(unpack_user* u PERL_UNUSED_DECL, const char* b PERL_UNUSED_DECL, const char* p, unsigned int l, SV** o)
 {
     dTHX;
     /*  newSVpvn(p, l) returns an undef if p == NULL */
     *o = ((l==0) ? newSVpvs("") : newSVpvn(p, l));
+    sv_utf8_decode(*o);
+    return 0;
+}
+
+STATIC_INLINE int template_callback_bin(unpack_user* u PERL_UNUSED_DECL, const char* b PERL_UNUSED_DECL, const char* p, unsigned int l, SV** o)
+{
+    dTHX;
+    *o = ((l==0) ? newSVpvs("") : newSVpvn(p, l));
     if(u->utf8) {
         sv_utf8_decode(*o);
     }
     return 0;
 }
 
+STATIC_INLINE int template_callback_ext(unpack_user* u PERL_UNUSED_DECL, const char* b PERL_UNUSED_DECL, const char* p PERL_UNUSED_DECL,
+                                        unsigned int l PERL_UNUSED_DECL, SV** o PERL_UNUSED_DECL)
+{
+    croak("EXT type is not supporeted yet");
+    return 0;
+}
+
 #include "msgpack/unpack_template.h"
 
 #define UNPACKER(from, name)                                                  \
@@ -287,8 +302,6 @@ XS(xs_unpack) {
     dXSARGS;
     SV* const self = ST(0);
     SV* const data = ST(1);
-    size_t limit;
-
     unpack_user u = UNPACK_USER_INIT;
 
     // setup configuration
@@ -302,13 +315,7 @@ XS(xs_unpack) {
         }
     }
 
-    if (items == 2) {
-        limit = sv_len(data);
-    }
-    else if(items == 3) {
-        limit = SvUVx(ST(2));
-    }
-    else {
+    if (!(items == 2 || items == 3)) {
         Perl_croak(aTHX_ "Usage: Data::MessagePack->unpack('data' [, $limit])");
     }
 
